;;; L-Systems -- general
;; Valvassori Mose -- 2001
;; $Id: general.scm,v 1.8 2001/09/21 20:36:41 djedi Exp $


;; constantes des cls dans la definition d'un lsys
(define type 'type)
(define w 'w)
(define prod 'prod)
(define name 'name)
(define author 'author)
(define description 'description)
(define angle 'angle)

(define PI 3.14159)
(define (deg->rad x)
  (/ (* x PI) 180))
(define (rad->deg x)
  (/ (* x 180) PI))


;; Donne la valeur d'une paire (key value)
(define-syntax lsys-value
  (syntax-rules ()
    ((lsys-value pair)
     (cadr pair))
    ))

;; remplace assq
;; renvoie directement la valeur ou fail si elle existe pas.
(define-syntax lsys-assq
  (syntax-rules ()
    ((lsys-assq field lsys fail)
     (let ((x (assq field lsys)))
       (if (list? x)
	   (lsys-value x)
	   fail)))
    ))

(define-syntax lsys-type
  (syntax-rules ()
    ((lsys-type lsys)
     (lsys-assq type lsys (error "Bad L-system" lsys)))))

(define-syntax lsys-w
  (syntax-rules ()
    ((lsys-w lsys)
     (lsys-assq w lsys "")))
    )

(define-syntax lsys-prod
  (syntax-rules ()
    ((lsys-prod lsys)
     (lsys-assq prod lsys "")))
    )


(define-syntax lsys-name
  (syntax-rules ()
    ((lsys-name lsys)
     (lsys-assq name lsys ""))
    ))

(define-syntax lsys-author
  (syntax-rules ()
    ((lsys-author lsys)
     (lsys-assq author lsys ""))
    ))

(define-syntax lsys-description
  (syntax-rules ()
    ((lsys-description lsys)
     (lsys-assq description lsys ""))
    ))

(define-syntax lsys-angle
  (syntax-rules ()
    ((lsys-angle lsys)
     (lsys-assq angle lsys 0))
))




(define lsys-random-seed 123456)
(define (lsys-random)
  (set! lsys-random-seed (modulo (* 3271 (+ lsys-random-seed 3)) 1000))
  (/ lsys-random-seed 1000))
(lsys-help-add-item 'lsys-random "(lsys-random)

Return a pseudo random number between 0.0 and 1.0
You can change the seed by
(define lsys-random-seed my-ssed)
")



;;; Wrapper qui orduit que la chaine
(define (lsys-gen lsys n)
  ((lsys-value (assq (lsys-type lsys) lsys-generator-list)) lsys n))
(lsys-help-add-item 'lsys-gen "(lsys-gen lsys n)

Generate n iter of this lsys")

;;; Wrappeur qui genere directement la chaine tortue si elle a lieu d'etre 
;; ou effectue directment la sortie.
(define (lsys-generate-turtle-string lsys n turtle)
  ((lsys-assq 'lsys2turtle turtle '()) (lsys-gen lsys n) lsys turtle) ;; lsys-turtle-ass (lsys-angle lsys) 5)
)
(lsys-help-add-item 'lsys-generate-turtle-string "(lsys-generate-turtle-string lsys n turtle-system)

Generate a complete turtle string")

;; La liste des generateurs pour la factory
;; table de la forme ((type-key lambda-generateur) ...)
(define lsys-generator-list '())

;; Ajoute un generateur dans la table des generateurs
;; type      : nom de generateur (dol,...)
;; generator : lambda du generateurs.
(define (lsys-add-generator type generator)
  (set! lsys-generator-list (cons (list type generator) lsys-generator-list)))
