;;; Stochastic OL-System
;; Valvassori Mose -- 2001
;; $Id: sol.scm,v 1.2 2001/09/21 20:36:41 djedi Exp $



;; lsys : dfinition du lsys
;; n :    nombre d'itrations
(define (lsys-sol lsys n)
  (if (not (eq? (lsys-type lsys) 'sol))
      (error "Not a S-OL system" lsys)

      (lsys-sol-product (lsys-w lsys) (lsys-prod lsys) n)
      ))
(lsys-add-generator 'sol lsys-sol)
(lsys-help-add-item 'lsys-sol "(lsys-sol)
Expand a Stochastic OL")

;; renvoie la probabilit
;; rule :  la rgle interne d'un sol (proba (production))
(define-syntax lsys-sol-rule-proba
  (syntax-rules ()
    ((lsys-sol-rule-proba rule)
     (car rule))))

;; renvoie la production
;; rule :  la rgle interne d'un sol (proba (production))
(define-syntax lsys-sol-rule-prod
  (syntax-rules ()
    ((lsys-sol-rule-proba rule)
     (cadr rule))))




;; Renvoie les regles associe a un symbole
;; prod : jeu de regle
;; symbol : le symbol que l'on cherche
(define-syntax lsys-sol-rule
  (syntax-rules ()
    ((lsys-sol-rule symbol prod)
     (lsys-assq symbol prod #f))))

;; Choisi une production selon ses chances
;; symbol : le symbole que l'on cherche
;; prule  : les regles de production
(define (lsys-sol-choose-rule symbol prule)
  ; (define (prob r) (caar r))
  ; (define (prod r) (cadar r))
  (define (aux p r)
    (let ((fisrt-rule (car r)))
      (if (< (- p (lsys-sol-rule-proba fisrt-rule)) 0)
	  (lsys-sol-rule-prod fisrt-rule)
	  (aux (- p (lsys-sol-rule-proba fisrt-rule)) (cdr r))
	  )))
  (let ((rule (lsys-sol-rule symbol prule)))
    (if (eq? #f rule) ; si le sombol n'est pas dans la table
	#f
	(aux (lsys-random) rule)
    )))




;; string : la chaine initiale
;; prules : les rgles de production
;; n      : le nombre d'itration qu'il reste  effectuer
(define (lsys-sol-product string prules n)
  (if (zero? n)
      string
      (lsys-sol-product (lsys-sol-next-iter (reverse string) prules '()) prules (- n 1))))

;; string : la chaine initiale
;; prules : les rgles de production
;; result : la chaine resultat
(define (lsys-sol-next-iter string prules result)
  (if (null? string)
      result
      (lsys-sol-next-iter (cdr string) prules
			  (append
			   (let ((prod (lsys-sol-choose-rule (car string) prules))) ; cherche la regle de prod
			  	(if (eq? prod #f)  ; si le sombol n'est pas dans la table
			  	    (list (car string))
			  	    prod )) ;(lsys-value prod)))
			   result)
			  )))
